<?php
// wcf imports
require_once(WCF_DIR.'lib/system/event/EventListener.class.php');
require_once(WCF_DIR.'lib/data/message/wqm/Quotes.class.php');

/**
 *
 * @author      Raul Grieben
 * @copyright   2011 Raul Grieben
 * @license     Creative Commons Namensnennung-NichtKommerziell-KeineBearbeitung 3.0 Deutschland Lizenz <http://creativecommons.org/licenses/by-nc-nd/3.0/de/>
 * @package     de.bissfan.wcf.wqm
 * @subpackage  system.event.listener
 * @category    Burning Board 
 */
class WQMBoardListener implements EventListener {
	public $type = 'board';
	
	/**
	 * @see EventListener::execute()
	 */
	public function execute($eventObj, $className, $eventName) {
		if (!MODULE_WQM) return;
				
		switch ($className)
		{
			case 'PostAddForm':
			case 'PostQuickAddForm':
				$this->addPost($eventObj->newPost->postID);
			break;
			case 'PostEditForm':
				$this->addPost($eventObj->post->postID);
			break;
			case 'ThreadAddForm':
				$this->addPost($eventObj->newThread->firstPostID);
			break;
			case 'WQMRedirectAction':
				$do = intval($_REQUEST['do']);
				if($this->type == $eventObj->row['type']){
					switch($do)
					{
						case 1: $eventObj->redirect .= 'Board&boardID='.$eventObj->row['firstID'].SID_ARG_2ND_NOT_ENCODED;
							break;
						case 2: $eventObj->redirect .= 'Thread&threadID='.$eventObj->row['secondID'].SID_ARG_2ND_NOT_ENCODED;
							break;
						case 3: $eventObj->redirect .= 'Thread&postID='.$eventObj->row['thirdID'].SID_ARG_2ND_NOT_ENCODED.'#post'.$eventObj->row['thirdID'];
							break;
						default:
							throw new IllegalLinkException();
						break;
					}
				}
			break;
			case 'ThreadPage':
				if(WCF::getUser()->userID > 0)
				{
					Quotes::markReadByThirdIDs(WCF::getUser(),$eventObj->postList->postIDs,$this->type);
					
					if(WQM_AUTOLINK_USER){
						foreach ($eventObj->postList->posts as $post) {
							$userIDs = Quotes::checkForQuote($post->subject, $post->message, true);
							foreach($userIDs as $user)
							{
								$post->message = str_replace('@'.$user['userName'].':', '@[url=\''.PAGE_URL.RELATIVE_WBB_DIR.'/index.php?page=User&userID='.$user['userID'].'\']'.$user['userName'].'[/url]', $post->message);
								$post->message = str_replace('@'.$user['userName'].'', '@[url=\''.PAGE_URL.RELATIVE_WBB_DIR.'/index.php?page=User&userID='.$user['userID'].'\']'.$user['userName'].'[/url]', $post->message);
								$post->message = str_replace('²'.$user['userName'].':', '²[url=\''.PAGE_URL.RELATIVE_WBB_DIR.'/index.php?page=User&userID='.$user['userID'].'\']'.$user['userName'].'[/url]', $post->message);
								$post->message = str_replace('²'.$user['userName'].'', '²[url=\''.PAGE_URL.RELATIVE_WBB_DIR.'/index.php?page=User&userID='.$user['userID'].'\']'.$user['userName'].'[/url]', $post->message);
								$post->messageCache = null;
							}
						}
					}	
				}
			break;	
			case 'ThreadActionPage':
				switch($eventObj->action)
				{
					case 'trash':
					case 'delete':
					case 'disable':
						$this->delThread($eventObj->threadID);
					break;
					
					case 'recover':
					case 'enable':
						$this->addThread($eventObj->threadID,true);
					break;
					
					case 'trashAll':
					case 'deleteAll':
					case 'disableAll':
						$this->delThread(WCF::getSession()->getVar('markedThreads'));
					break;
					
					case 'recoverAll':
					case 'enableAll':
						$this->addThread(WCF::getSession()->getVar('markedThreads'),true);
					break;	
					
				}
			break;

			case 'PostActionPage':
				switch($eventObj->action)
				{
					case 'trash':
					case 'delete':
					case 'disable':
						$this->delPost($eventObj->postID);
					break;
					
					case 'recover':
					case 'enable':
						$this->addPost($eventObj->postID,true);
					break;
					
					case 'trashAll':
					case 'deleteAll':
					case 'disableAll':
						$this->delPost(WCF::getSession()->getVar('markedPosts'));
					break;
					
					case 'recoverAll':
					case 'enableAll':
						$this->addPost(WCF::getSession()->getVar('markedPosts'),true);
					break;	
					
				}
			break;
		}
	}
	
	public function checkPermission($userID, $boardID){
		require_once(WBB_DIR.'lib/data/user/WBBUserSession.class.php');
        $user = new WBBUserSession($userID);
        return $user->getBoardPermission('canReadThread', $boardID);
	}
	
	public function addPost($postIDs, $mod = false, $thread = false){
		
		require_once(WBB_DIR.'lib/data/post/PostEditor.class.php');
		require_once(WBB_DIR.'lib/data/thread/ThreadEditor.class.php');
		require_once(WBB_DIR.'lib/data/board/BoardEditor.class.php');
		
		if(!is_array($postIDs))
		{
			$postIDs = array(intval($postIDs));	
		}
		
		foreach($postIDs as $postID)
		{
			if($postID <= 0)return;
			$post = new PostEditor($postID);
			if($post->threadID <= 0)return;
			$thread = new ThreadEditor($post->threadID);
			if($thread->boardID <= 0)return;
			$board = new BoardEditor($thread->boardID);
	
			$can_post = false;
			
			if($thread  &&  WCF::getUser()->getBoardPermission('canStartThreadWithoutModeration', $thread->boardID))$can_post = true;
			if(!$thread &&  WCF::getUser()->getBoardPermission('canReplyThreadWithoutModeration', $thread->boardID))$can_post = true;
			
			if($mod)$can_post = true;
			
			if ($can_post)
			{
				$subject = ($post->subject == '') ? $thread->topic : $post->subject;		
				$userIDs = Quotes::checkForQuote($post->subject, $post->message);
				Quotes::deleteByNotInUserIDs(implode(',', $userIDs), $post->postID, $this->type);
				foreach($userIDs as $userID){
					if($this->checkPermission($userID, $board->boardID)){
					   Quotes::addQuote($userID, 
									   $post->userID, 
									   $post->username, 
									   $subject, 
									   $thread->topic, 
									   $post->message, 
									   $board->boardID, 
									   $thread->threadID, 
									   $post->postID, 
									   $board->title, 
									   $thread->topic, 
									   $post->time, 
									   $this->type);	
					}
				}
			}
		}
	}
	
	public function delPost($postIDs){

		if(!is_array($postIDs))
		{
			$postIDs = array(intval($postIDs));	
		}
		
		Quotes::deleteByThirdIDs(implode(',', $postIDs),$this->type);
	}
	
	public function addThread($threadIDs, $mod = false){
		
		require_once(WBB_DIR.'lib/data/thread/ThreadEditor.class.php');
		
		if(!is_array($threadIDs))
		{
			$threadIDs = array(intval($threadIDs));	
		}
		
		foreach($threadIDs as $threadID)
		{
			$thread = new ThreadEditor($threadID);
			$this->addPost(explode(',',$thread->getPostIDs()), $mod, true);
		}
	}
	
	public function delThread($threadIDs){

		if(!is_array($threadIDs))
		{
			$threadIDs = array(intval($threadIDs));	
		}
		
		Quotes::deleteBySecondIDs(implode(',', $threadIDs),$this->type);
	}
}
?>